#ifdef LINUX
#include <endian.h>      /* so that sndlib.h will get host byte-order right */
#endif
#ifdef SGI
#include <sys/endian.h>
#endif
#include "sndlib.h"
#include "byte_routines.h"

/* for reading and writing headers */
int sndlib_read_header(int);
int sndlib_write_header(int, int, int, int, int, int, char *, int *);
int sndlib_set_header_data_size(int, int, int);


/* some handy macros */

#define IS_FLOAT_FORMAT(format) (                           \
              (format) == SNDLIB_32_FLOAT                   \
           || (format) == SNDLIB_32_FLOAT_LITTLE_ENDIAN     )

#define NOT_A_SOUND_FILE(header_type) (                     \
              (header_type) == unsupported_sound_file       \
           || (header_type) == raw_sound_file               )

/* The header types that sndlib can write, as of sndlib-5.5. */
#define WRITEABLE_HEADER_TYPE(type) (                       \
               (type) == AIFF_sound_file                    \
           ||  (type) == NeXT_sound_file                    \
           ||  (type) == RIFF_sound_file                    \
           ||  (type) == IRCAM_sound_file                   )

#define INVALID_DATA_FORMAT(format) ((format) < 1)

/* The data formats that disk-based cmix code can read and write. */
#define SUPPORTED_DATA_FORMAT(format) (                     \
               (format) == SNDLIB_16_LINEAR                 \
           ||  (format) == SNDLIB_16_LINEAR_LITTLE_ENDIAN   \
           ||  (format) == SNDLIB_32_FLOAT                  \
           ||  (format) == SNDLIB_32_FLOAT_LITTLE_ENDIAN    )

/* Usually endianness is handled for us by sndlib, but there are some
   cases where it's helpful to know (e.g., sfcreate). These are the
   more common formats sndlib supports -- not all of them.
*/
#define IS_BIG_ENDIAN_FORMAT(format) (                      \
              (format) == SNDLIB_16_LINEAR                  \
           || (format) == SNDLIB_32_FLOAT                   \
           || (format) == SNDLIB_24_LINEAR                  \
           || (format) == SNDLIB_32_LINEAR                  \
           || (format) == SNDLIB_16_UNSIGNED                )

#define IS_LITTLE_ENDIAN_FORMAT(format) (                   \
              (format) == SNDLIB_16_LINEAR_LITTLE_ENDIAN    \
           || (format) == SNDLIB_32_FLOAT_LITTLE_ENDIAN     \
           || (format) == SNDLIB_24_LINEAR_LITTLE_ENDIAN    \
           || (format) == SNDLIB_32_LINEAR_LITTLE_ENDIAN    \
           || (format) == SNDLIB_16_UNSIGNED_LITTLE_ENDIAN  )

#define FORMATS_SAME_BYTE_ORDER(format1, format2) (         \
             ( IS_BIG_ENDIAN_FORMAT(format1)                \
            && IS_BIG_ENDIAN_FORMAT(format2) )              \
         ||  ( IS_LITTLE_ENDIAN_FORMAT(format1)             \
            && IS_LITTLE_ENDIAN_FORMAT(format2) )           )

